@@ -28,15 +28,17 @@ def check_latest_version():
     try:
         is_outdated, latest_version = check_outdated(__name__, __version__)
         if is_outdated:
-            click.echo(
-                f'Your local version ({__version__}) is out of date! Latest is {latest_version}!'
+            click.secho(
+                f'Your local version ({__version__}) is out of date! Latest is {latest_version}!',
+                bold=True, file=sys.stderr,
             )
     except ValueError:
         # this happens when your local version is ahead of the pypi version,
         # which happens only in development
         pass
     except KeyError:
-        click.secho('Could not determine package version, continue nevertheless.', bold=True, file=sys.stderr)
+        # this happens when in dev mode, message could be removed
+        click.secho('Could not determine package version, continue nevertheless.', file=sys.stderr)
         pass
 
 CLICK_CONTEXT_SETTINGS = dict(help_option_names=['-h', '--help'])
@@ -45,7 +47,7 @@ TG_COMMANDS=[
     'init', 'validate', 'validate-inputs', 'plan', 'apply',
     'destroy', 'info', 'output', 'show'
     ]
-
+STAGES=['global', 'sbx', 'dev', 'qas', 'run', 'tst', 'acc', 'prd']
 class DefaultGroup(click.Group):
     '''
     Allow a default command for a group
@@ -209,6 +211,53 @@ def run_all(command, verbose, debug, dry_run, no_lock, update_source, exclude_ex
         terragrunt_args=terragrunt_args,
     )
 
+@main.command(
+    name="import",
+    context_settings=dict(
+        ignore_unknown_options=True,
+    ),
+)
+@click.option(
+    '--address', '-a', required=True,
+    help='Terraform resource address',
+    )
+@click.option(
+    '--id', '-i', required=True,
+    help='Id of the resource to be imported',
+    )
+@click.option('--verbose', '-v', is_flag=True, default=False,
+    help='Verbose printing',
+    show_default=True
+    )
+@click.option('--dry-run', '-D', is_flag=True, default=False,
+    help='dry-run mode, no real actions are executed',
+    show_default=True
+    )
+@click.option(
+    '--working-dir', '-w', default=None,
+    help='Working directory, when omitted the current directory is used',
+    )
+@click.option('--no-lock', '-n', is_flag=True, default=False,
+    help='Do not apply a lock while executing the command (only applicable with plan)',
+    show_default=True
+    )
+@click.argument('terragrunt-args', nargs=-1, type=click.UNPROCESSED)
+@click.version_option(version=__version__)
+def run_import(address, id, verbose, dry_run, working_dir, no_lock, terragrunt_args):
+    """ Executes the terragrunt/terraform import command """
+
+    check_latest_version()
+
+    tgwrap = TgWrap(verbose=verbose)
+    tgwrap.run_import(
+        address=address,
+        id=id,
+        dry_run=dry_run,
+        working_dir=working_dir,
+        no_lock=no_lock,
+        terragrunt_args=terragrunt_args,
+    )
+
 @main.command(
     name="analyze",
     context_settings=dict(
@@ -245,3 +294,101 @@ def analyze(verbose, dry_run, exclude_external_dependencies, working_dir, terrag
         working_dir=working_dir,
         terragrunt_args=terragrunt_args,
     )
+
+@main.command(
+    name="promote",
+    context_settings=dict(
+        ignore_unknown_options=True,
+    ),
+)
+@click.option(
+    '--source-stage', '-s', required=True,
+    type=click.Choice(STAGES, case_sensitive=True),
+    help='Source of config files',
+    )
+@click.option(
+    '--target-stage', '-t', required=True,
+    type=click.Choice(STAGES, case_sensitive=True),
+    help='Target of config files',
+    )
+@click.option('--clean-up', '-c', is_flag=True, default=False,
+    help='Clean up files on target side that do not exist as source',
+    show_default=True
+    )
+@click.option('--verbose', '-v', is_flag=True, default=False,
+    help='Verbose printing',
+    show_default=True
+    )
+@click.option('--dry-run', '-D', is_flag=True, default=False,
+    help='dry-run mode, no real actions are executed',
+    show_default=True
+    )
+@click.option(
+    '--working-dir', '-w', default=None,
+    help='Working directory, when omitted the current directory is used',
+    )
+@click.version_option(version=__version__)
+def promote(verbose, dry_run, clean_up, working_dir, source_stage, target_stage):
+    """ Promotes the terragrunt config files from one stage to another """
+
+    check_latest_version()
+
+    tgwrap = TgWrap(verbose=verbose)
+    tgwrap.promote(
+        dry_run=dry_run,
+        clean_up=clean_up,
+        working_dir=working_dir,
+        source_stage=source_stage,
+        target_stage=target_stage,
+    )
+
+@main.command(
+    name="deploy",
+    context_settings=dict(
+        ignore_unknown_options=True,
+    ),
+)
+@click.option(
+    '--source-domain', '-d', required=True,
+    help='Source domain of config files',
+    )
+@click.option(
+    '--source-stage', '-s', required=True,
+    type=click.Choice(STAGES, case_sensitive=True),
+    help='Source stage of config files',
+    )
+@click.option(
+    '--target-domain', '-t', required=True,
+    help='Target domain of config files',
+    )
+@click.option('--clean-up', '-c', is_flag=True, default=False,
+    help='Clean up files on target side that do not exist as source',
+    show_default=True
+    )
+@click.option('--verbose', '-v', is_flag=True, default=False,
+    help='Verbose printing',
+    show_default=True
+    )
+@click.option('--dry-run', '-D', is_flag=True, default=False,
+    help='dry-run mode, no real actions are executed',
+    show_default=True
+    )
+@click.option(
+    '--working-dir', '-w', default=None,
+    help='Working directory, when omitted the current directory is used',
+    )
+@click.version_option(version=__version__)
+def deploy(verbose, dry_run, clean_up, working_dir, source_domain, source_stage, target_domain):
+    """ Deploys the terragrunt config files from one domain/state to another """
+
+    check_latest_version()
+
+    tgwrap = TgWrap(verbose=verbose)
+    tgwrap.deploy(
+        dry_run=dry_run,
+        clean_up=clean_up,
+        working_dir=working_dir,
+        source_domain=source_domain,
+        source_stage=source_stage,
+        target_domain=target_domain,
+    )
