@@ -16,8 +16,9 @@ import os
 import sys
 import subprocess
 import shlex
-import json
+import shutil
 import re
+import json
 import click
 import networkx as nx
 
@@ -80,6 +81,10 @@ class TgWrap():
                 f"No '{env_var}' variable is set, so the sources as defined in terragrunt.hcl files will be used as is!"
                 )
 
+    def is_installed(self, program):
+        """ Checks if a program is installed on the system """
+        return shutil.which(program) is not None
+
     def prepare_groups(self, graph, exclude_external_dependencies):
         """ Prepare the list of groups that will be executed """
         groups = []
@@ -225,7 +230,7 @@ class TgWrap():
 
         lock_stmt         = '-lock=false' if no_lock else ''
         update_stmt       = '--terragrunt-source-update' if update_source else ''
-        ignore_deps_stmt  = '--terragrunt-ignore-external-dependencies' if exclude_external_dependencies else ''
+        ignore_deps_stmt  = '--terragrunt-ignore-external-dependencies' if exclude_external_dependencies else '--terragrunt-include-external-dependencies'
         debug_stmt        = '--terragrunt-log-level debug --terragrunt-debug' if debug else ''
         auto_approve_stmt = '--terragrunt-no-auto-approve' if no_auto_approve else ''
         interactive_stmt  = '--terragrunt-non-interactive' if non_interactive else ''
@@ -260,7 +265,7 @@ class TgWrap():
         auto_approve, working_dir, terragrunt_args):
         """ Executes a terragrunt command on a single project """
 
-        self.printer.verbose(f"Attempting to 'run {command}'")
+        self.printer.verbose(f"Attempting to execute 'run {command}'")
         if terragrunt_args:
             self.printer.verbose(f"- with additional parameters: {' '.join(terragrunt_args)}")
 
@@ -287,14 +292,17 @@ class TgWrap():
         if dry_run:
             self.printer.warning(f'In dry run mode, no real actions are executed!!')
         else:
-            rc = subprocess.run(shlex.split(cmd))
+            # the `posix=False` is to prevent the split command to remove quotes from strings,
+            # e.g. when executing commands like this:
+            # tgwrap state mv 'azuread_group.this["viewers"]' 'azuread_group.this["readers"]'
+            rc = subprocess.run(shlex.split(cmd, posix=False))
             self.printer.verbose(rc)
 
     def run_all(self, command, debug, dry_run, no_lock, update_source,
         exclude_external_dependencies, step_by_step, working_dir, terragrunt_args):
         """ Executes a terragrunt command across multiple projects """
 
-        self.printer.verbose(f"Attempting to 'run-all {command}'")
+        self.printer.verbose(f"Attempting to execute 'run-all {command}'")
         if terragrunt_args:
             self.printer.verbose(f"- with additional parameters: {' '.join(terragrunt_args)}")
 
@@ -327,6 +335,51 @@ class TgWrap():
                 rc = subprocess.run(shlex.split(cmd))
                 self.printer.verbose(rc)
 
+    def run_import(self, address, id, dry_run, working_dir, no_lock, terragrunt_args):
+        """ Executes the terragrunt/terraform import command """
+
+        self.printer.verbose(f"Attempting to execute 'run import'")
+        if terragrunt_args:
+            self.printer.verbose(f"- with additional parameters: {' '.join(terragrunt_args)}")
+
+        check_for_file="terragrunt.hcl"
+        if working_dir:
+            check_for_file = os.path.join(working_dir, check_for_file)
+        if not os.path.isfile(check_for_file):
+            self.printer.error(
+                f"{check_for_file} not found, this seems not to be a terragrunt project directory!"
+                )
+            sys.exit(1)
+
+        lock_stmt         = '-lock=false' if no_lock else ''
+        working_dir_stmt  = f'--terragrunt-working-dir {working_dir}' if working_dir else ''
+
+        cmd = f"terragrunt import {working_dir_stmt} {lock_stmt} {address} {id} {' '.join(terragrunt_args)}"
+        cmd = re.sub(' +', ' ', cmd)
+        self.printer.verbose(f'Full command to execute:\n$ {cmd}')
+
+        # cmd = 'echo hello world' # just testing
+
+        if dry_run:
+            self.printer.warning(f'In dry run mode, no real actions are executed!!')
+        else:
+            env = os.environ.copy()
+            # TERRAGRUNT_SOURCE should not be present (or it should be a fully qualified path (which is typically not the case))
+            value = env.pop('TERRAGRUNT_SOURCE')
+            if value:
+                self.printer.verbose(
+                    f'Terragrunt source environment variable with value {value} will be ignored'
+                    )
+
+            # the `posix=False` is to prevent the split command to remove quotes from strings,
+            # e.g. when executing commands like this:
+            # tgwrap import 'azuread_group.this["viewers"]' '123e4567-e89b-12d3-a456-426655440000'
+            rc = subprocess.run(
+                shlex.split(cmd, posix=False),
+                env=env,
+            )
+            self.printer.verbose(rc)
+
     def analyze(self, dry_run, exclude_external_dependencies, working_dir, terragrunt_args):
         """ Analyzes the plan files """
 
@@ -417,3 +470,69 @@ class TgWrap():
 
         if not ts_validation_successful:
             raise Exception("Terrasafe validation failed on one or more projects")
+
+    def promote(self, dry_run, clean_up, working_dir, source_stage, target_stage):
+        """ Promotes the terragrunt config files from one stage to another """
+
+        # do we have a working dir?
+        working_dir = working_dir if working_dir else os.getcwd()
+        source_path = os.path.join(working_dir, source_stage, '')
+        target_path = os.path.join(working_dir, target_stage, '')
+
+        # Check if source exists
+        if not self.is_installed('rsync'):
+            self.printer.error("'rsync' seems not installed. Cannot continue")
+        elif not os.path.exists(source_path):
+            self.printer.error(f"Could not find stage {source_stage} in path {working_dir}. Cannot continue.")
+        else:
+            self.printer.verbose(f"Copying config: {source_path} => {target_path}")
+
+            dry_run_stmt = '--dry-run' if dry_run else ''
+            clean_stmt   = '--delete' if clean_up else ''
+        
+            cmd = f"rsync -aim {dry_run_stmt} {clean_stmt}" + \
+                "--include='terragrunt.hcl' --include='.terraform.lock.hcl' " + \
+                "--exclude='env.hcl' --exclude='.terragrunt-cache/' " + \
+                "--exclude='terragrunt-debug.tfvars.json' --exclude=planfile " + \
+                f"{source_path} {target_path}"
+
+            cmd = re.sub(' +', ' ', cmd)
+
+            rc = subprocess.run(shlex.split(cmd))
+            self.printer.verbose(rc)
+
+    def deploy(self, dry_run, clean_up, working_dir, source_domain, source_stage, target_domain):
+        """ Deploys the terragrunt config files from one domain/state to another """
+    
+        # do we have a working dir?
+        working_dir = working_dir if working_dir else os.getcwd()
+        source_path = os.path.join(working_dir, source_domain, source_stage, '')
+        target_path = os.path.join(working_dir, target_domain, source_stage, '')
+
+        # Check if source exists
+        if not self.is_installed('rsync'):
+            self.printer.error("'rsync' seems not installed. Cannot continue")
+        elif not os.path.exists(source_path):
+            self.printer.error(f"Could not find stage {source_stage} in path {working_dir}. Cannot continue.")
+        else:
+            self.printer.verbose(f"Copying config: {source_path} => {target_path}")
+
+            try:
+                os.makedirs(target_path)
+            except OSError:
+                # directory already exists
+                pass
+
+            dry_run_stmt = '--dry-run' if dry_run else ''
+            clean_stmt   = '--delete' if clean_up else ''
+
+            cmd = f"rsync -aim {dry_run_stmt} {clean_stmt} " + \
+                "--include='terragrunt.hcl' --include='.terraform.lock.hcl' " + \
+                "--exclude='env.hcl' --exclude='.terragrunt-cache/' " + \
+                "--exclude='terragrunt-debug.tfvars.json' --exclude=planfile " + \
+                f"{source_path} {target_path}"
+
+            cmd = re.sub(' +', ' ', cmd)
+
+            rc = subprocess.run(shlex.split(cmd))
+            self.printer.verbose(rc)
